uniform mat4 u_mvp;

#ifdef ANIMATE_UV_USER
  uniform float u_time_user;  
#endif

#ifdef ANIMATE_UV
  uniform float u_time;
#endif

#if defined(ANIMATE_UV) || defined(ANIMATE_UV_USER)
  uniform vec2  u_time_speed_uv;
#endif  
  
#ifdef FAKE_LIGHT
  uniform mat4 u_m;
  const float modifier = 0.3;
  const vec4 color_add = vec4(0.710, 0.533, 0.036, 0.0);
#endif

#ifdef TERRAIN_LIGHT
  const vec4 color_add = vec4(0.710, 0.533, 0.036, 0.0);
  const float modifier = 0.4;
#endif  
 
#ifdef WOBBLE
  uniform float       u_freq;
  uniform float       u_amplitude;  
  const float c_2PI = 6.28318; 
#endif  
 
void main()
{ 
  #ifdef SKINNING
    vec4 positionL = CalculateSkinPositionL(a_position, a_blendweight, a_blendindex);
  #else
    vec4 positionL = a_position;
  #endif
  
  #ifdef USE_UV0
    v_uv0 = a_uv0;
  #endif
  
  #ifdef USE_UV1
    v_uv1 = a_uv1;
  #endif
  
  #ifdef USE_UV2
    v_uv2 = a_uv2;
  #endif
  
  #ifdef USE_UV3
    v_uv3 = a_uv3;
  #endif

  #if defined(USE_COLOR0)
    v_color0 = a_color0;
  #endif
  
  ///////////////////////////////////////////////////////////////
  // UV Animation
  ///////////////////////////////////////////////////////////////
  #if defined(ANIMATE_UV_USER) || defined(ANIMATE_UV)
    #if defined(ANIMATE_UV_USER)
      float time = u_time_user;
    #elif defined(ANIMATE_UV)
      float time = u_time;
    #endif
    
     v_uv0 = AnimateUV(a_uv0, time, u_time_speed_uv);      
  #endif

  #ifdef FAKE_LIGHT
    vec4 positionW = positionL * u_m;
    positionW.y    = 0.0;
  
    float factor    = modifier  * clamp(1.0 - length(positionW.xyz) / 1700.0, 0.0, 1.0);
    v_color0        = color_add * factor;
  #endif
  
  #ifdef TERRAIN_LIGHT
    float factor    = modifier * clamp(1.0 - length(a_position.xyz) / 110.0, 0.0, 1.0);
    v_color0        = color_add * factor;
  #endif
  
  #ifdef WOBBLE    
    float distance  = length(positionL.xz);  
    positionL.y     += u_amplitude * sin(c_2PI * u_time * u_freq + distance) * a_color0.w;
  #endif
  
  gl_Position = positionL * u_mvp;  
}