#include "VertexFormatDefine.shader"
#include "FragmentFormatDefine.shader"
#include "Common.shader"

uniform mat4  u_mvp;
uniform mat4  u_m;
uniform vec4  u_foam_pos[4];
uniform float u_foam_height;
uniform float u_foam_curve_factor;

varying float rel_height;

void main()
{ 
  // Set all attributes to temporary variables e.g internal_color1 <- a_color1
  #include "VertexShaderPrologue.shader"
  
  #ifdef VTX_SKIN
    internal_positionW = CalculateSkinPositionW(internal_positionL, a_blendweight, a_blendindex);
  #else 
    internal_positionW = internal_positionL * u_m;
  #endif

  float distArray[4];
  distArray[0] = length(internal_positionW.xz - u_foam_pos[0].xz);
  distArray[1] = length(internal_positionW.xz - u_foam_pos[1].xz);
  distArray[2] = length(internal_positionW.xz - u_foam_pos[2].xz);
  distArray[3] = length(internal_positionW.xz - u_foam_pos[3].xz);
  
  float avg_count = 0.0;   
  rel_height = 0.0;
  for(int i=0;i<4;++i)
  {      
    float dist      = distArray[i];   
    float maxHeight = u_foam_curve_factor * dist * dist + u_foam_height;
    if(maxHeight > 0.0)  
    {
      rel_height       += 1.0 - clamp(internal_positionW.y / maxHeight, 0.0, 1.0);
      avg_count       += 1.0;
    }
  }
    
  if(avg_count > 0.0)
  {
    // Average the height
    rel_height *= (1.0 / avg_count);
  }
    
  internal_positionF = internal_positionW * u_vp;  
  
  #include "VertexShaderEpilogue.shader"
  gl_Position = positionF;
}