precision highp float;

const int   c_zero 	    = 0;
const int   c_one 	    = 1;
const int   c_two 	    = 2;
const int   c_three     = 3;
const int   c_four 	    = 4;
const float c_stripemul   = 20.0;
const float c_stripoffset = 0.0;
const float c_half        = 0.5;
const mat2  c_rot         = mat2( 0.707168, -0.707168, 0.707168, 0.707168);

uniform vec4  u_palette[128];
uniform mat4  u_m;
uniform float u_time;

attribute vec4 a_position;
attribute vec4 a_blendindex;
attribute vec4 a_blendweight;
attribute vec2 a_uv0;

varying vec2  v_uv0;
varying vec2  v_uv1;
varying float v_lerpfac;

vec4 SkinPosition(in vec4 position, in int index, in float weight)
{
  vec4 temp;
  temp.x = dot(position, u_palette[c_four + index + c_zero]);
  temp.y = dot(position, u_palette[c_four + index + c_one]);
  temp.z = dot(position, u_palette[c_four + index + c_two]);
  temp.w = 0.0;
  return weight * temp;
}

void main()
{
  vec4 skin_position = vec4(0.0, 0.0, 0.0, 1.0);

  int index0   = int(a_blendindex[0]) * c_three;  
	  
  float weight0 = a_blendweight[0];  
	skin_position += SkinPosition(a_position, index0, weight0);
  
  int index1 	= int(a_blendindex[1]) * c_three;
  float weight1 = a_blendweight[1];  
	skin_position += SkinPosition(a_position, index1, weight1);
  
  int index2 	= int(a_blendindex[2]) * c_three;
  float weight2 = a_blendweight[2];  
	skin_position += SkinPosition(a_position, index2, weight2);
  
  int index3 	= int(a_blendindex[3]) * c_three;
  float weight3 = a_blendweight[3];  
	skin_position += SkinPosition(a_position, index3, weight3);
  
  v_uv0 = ((skin_position * u_m).xy + 1900.0) / 3800.0;  
  v_uv0 *= c_stripemul;
  v_uv1 = v_uv0 + c_stripoffset;     
  v_lerpfac = sin(u_time) * 0.5 + 0.5;
     
  v_uv0 = c_rot * v_uv0;
  v_uv1 = c_rot * v_uv1;
 
  mat4 u_mvp    = mat4(u_palette[c_zero], u_palette[c_one], u_palette[c_two], u_palette[c_three]);
  skin_position = skin_position * u_mvp;
 
  gl_Position = skin_position;  
}
