precision highp float;

const int c_zero 	      = 0;
const int c_one 	      = 1;
const int c_two 	      = 2;
const int c_three       = 3;
const int c_four 	      = 4;

// 128 vec4 
// 4 u_mvp + 40*3 matrix palette = 124 vec4 uniforms used
uniform vec4  u_palette[124];
// 1 vec4 uniform
uniform float u_outline_factor;
uniform vec4  u_camera;
// 4 vec4 uniform
uniform mat4  u_m;
// 4 vec4 uniform
uniform mat4  u_vp;

attribute vec4 a_position;
attribute vec4 a_blendindex;
attribute vec4 a_blendweight;
attribute vec2 a_uv0;
attribute vec4 a_normal;

varying vec2 v_uv0;
varying vec4 v_normal;

vec4 SkinTransform(in vec4 position, in int index, in float weight)
{
  vec4 temp;
  temp.x = dot(position, u_palette[c_four + index + c_zero]);
  temp.y = dot(position, u_palette[c_four + index + c_one]);
  temp.z = dot(position, u_palette[c_four + index + c_two]);
  temp.w = 0.0;
  return weight * temp;
}

void main()
{
  // Dynamic branching in shaders is generally not a good idea, but on some platforms
  // the computational cost is larger then branch

  vec4 skin_position = vec4(0.0, 0.0, 0.0, 1.0);
  vec4 skin_normal   = vec4(0.0, 0.0, 0.0, 1.0);
  
  int index0    = int(a_blendindex[0]) * c_three;  
  float weight0 = a_blendweight[0];

  skin_position += SkinTransform(a_position, index0, weight0);
  skin_normal   += SkinTransform(vec4(a_normal.xyz, 0.0), index0, weight0);

  
  int index1 	  = int(a_blendindex[1]) * c_three;
  float weight1 = a_blendweight[1];

  skin_position += SkinTransform(a_position, index1, weight1);
  skin_normal   += SkinTransform(vec4(a_normal.xyz, 0.0), index1, weight1);

  
  int index2 	  = int(a_blendindex[2]) * c_three;
  float weight2 = a_blendweight[2];

  skin_position += SkinTransform(a_position, index2, weight2);
  skin_normal += SkinTransform(vec4(a_normal.xyz, 0.0), index2, weight2);

  
  int index3 	  = int(a_blendindex[3]) * c_three;
  float weight3 = a_blendweight[3];

  skin_position += SkinTransform(a_position, index3, weight3);
  skin_normal   += SkinTransform(vec4(a_normal.xyz, 0.0), index3, weight3);
  
  v_uv0 = a_uv0;
  
  mat4 u_mvp = mat4(u_palette[c_zero], u_palette[c_one], u_palette[c_two], u_palette[c_three]);
  
  vec4 temp       = skin_position * u_mvp;  
  skin_position   = skin_position * u_m;
  
    
  skin_normal.w   = 0.0;
  skin_normal     = skin_normal * u_m;  
  skin_normal.xyz = normalize(skin_normal.xyz);
  
  vec4 toCamera   = vec4((u_camera - skin_position).xyz, 0.0);
  float len       = clamp(length(toCamera), 1500.0, 13000.0);
  
  skin_position   = skin_position + skin_normal * (len * 0.0003);
  skin_position   = skin_position * u_vp;
  gl_Position     = skin_position;
 
  v_normal = skin_normal;
}
