precision highp float;

const int c_zero 	= 0;
const int c_one 	= 1;
const int c_two 	= 2;
const int c_three = 3;
const int c_four 	= 4;


uniform vec4  u_palette[128];
uniform mat4  u_m;
uniform vec4  u_foam_pos[4];
uniform float u_foam_height;
uniform float u_foam_curve_factor;

attribute vec4 a_position;
attribute vec4 a_blendindex;
attribute vec4 a_blendweight;
attribute vec2 a_uv1;

varying vec2 v_uv0;
varying float rel_height;

vec4 SkinPosition(in vec4 position, in int index, in float weight)
{
  vec4 temp;
  temp.x = dot(position, u_palette[c_four + index + c_zero]);
  temp.y = dot(position, u_palette[c_four + index + c_one]);
  temp.z = dot(position, u_palette[c_four + index + c_two]);
  temp.w = 0.0;
  return weight * temp;
}

void main()
{
  vec4 skin_position = vec4(0.0, 0.0, 0.0, 1.0);

  int index0   = int(a_blendindex[0]) * c_three;  
	  
  float weight0 = a_blendweight[0];
	skin_position += SkinPosition(a_position, index0, weight0);
  
  int index1 	= int(a_blendindex[1]) * c_three;
  float weight1 = a_blendweight[1];
	skin_position += SkinPosition(a_position, index1, weight1);
  
  int index2 	= int(a_blendindex[2]) * c_three;
  float weight2 = a_blendweight[2];
	skin_position += SkinPosition(a_position, index2, weight2);
  
  int index3 	= int(a_blendindex[3]) * c_three;
  float weight3 = a_blendweight[3];
	skin_position += SkinPosition(a_position, index3, weight3);
  
  v_uv0 = a_uv1;
  
  vec4 skin_positionW = skin_position * u_m;
  mat4 u_mvp          = mat4(u_palette[c_zero], u_palette[c_one], u_palette[c_two], u_palette[c_three]);
  skin_position       = skin_position * u_mvp;

  float c_avg[5];
  c_avg[0] = 1.0; c_avg[1] = 1.0; c_avg[2] = 0.5; c_avg[3] = 0.33; c_avg[4] = 0.25;
  
  float distArray[4];
  distArray[0] = length(skin_positionW.xz - u_foam_pos[0].xz);
  distArray[1] = length(skin_positionW.xz - u_foam_pos[1].xz);
  distArray[2] = length(skin_positionW.xz - u_foam_pos[2].xz);
  distArray[3] = length(skin_positionW.xz - u_foam_pos[3].xz);
  
  float avg_count = 0.0;  
  rel_height         = 0.0;  
  for(int i=0;i<4;++i)
  {   
    float dist      = distArray[i];   
    float maxHeight = u_foam_curve_factor * dist * dist + u_foam_height;
    if(maxHeight > 0.0)  
    {
      rel_height       += 1.0 - clamp(skin_positionW.y / maxHeight, 0.0, 1.0);
      avg_count       += 1.0;
    }    
  }
  
  if(avg_count > 0.0)
  {
    // Average the height
    rel_height *= (1.0 / avg_count);
  }
  
  gl_Position   = skin_position;  
}