precision highp float;

varying vec4 v_color0;
varying vec4 v_color_add;
varying vec2 v_uv0;

uniform sampler2D s_texture0;
uniform sampler2D s_texture1;
uniform sampler2D s_texture2;
uniform sampler2D s_texture3;

// Gamma parameters
const float gamma_inv     = 1.0 / 2.2;
const vec3  gamma_inv_vec = vec3(gamma_inv, gamma_inv, gamma_inv);

void main()
{  
  vec4 tex_color0  = texture2D(s_texture0, v_uv0);
  vec4 tex_color1  = texture2D(s_texture1, v_uv0);
  vec4 tex_color2  = texture2D(s_texture2, v_uv0);  
  vec4 tex_color3  = texture2D(s_texture3, v_uv0);

  // Optimization note: 
  // -storing in linear space direcly would eliminate this step
  // -instead of using pow at the end, we can encode the function result inside a 1D texture
 
  vec4 result  = vec4( tex_color0.r  * v_color0.r + 
                       tex_color1.r  * v_color0.g + 
                       tex_color2.r  * v_color0.b + 
                       tex_color3.r  * v_color0.a,
  
                       tex_color0.g  * v_color0.r + 
                       tex_color1.g  * v_color0.g + 
                       tex_color2.g  * v_color0.b + 
                       tex_color3.g  * v_color0.a,

                       tex_color0.b  * v_color0.r + 
                       tex_color1.b  * v_color0.g + 
                       tex_color2.b  * v_color0.b + 
                       tex_color3.b  * v_color0.a, 

                       1.0);

  result += v_color_add;                       
                       
  // Convert to sRGB space
  result.rgb   = pow(result.rgb, gamma_inv_vec);
  gl_FragColor = result;
}